#!/usr/bin/python
import sys
import boto.ec2.elb
from charmhelpers.core import hookenv

# load are config every time!
# @HACK but not really
charm_config = hookenv.config()

elb = boto.ec2.elb.connect_to_region(
  charm_config['elb_region'],
  aws_access_key_id=charm_config['aws_access_key'],
  aws_secret_access_key=charm_config['aws_secret_key']
)

def getAvailabilityZoneList():
  # @TODO load from config, split and validate
  zones = charm_config['elb_availability_zones'].split(",")

  return zones

def getEC2():
  ec2 = boto.ec2.connect_to_region(
    charm_config['elb_region'],
    aws_access_key_id=charm_config['aws_access_key'],
    aws_secret_access_key=charm_config['aws_secret_key']
  )
  return ec2

def findOrCreateELB(name):
  lb = None
  try:
    elbs = elb.get_all_load_balancers(load_balancer_names=[name])
    if len(elbs) == 1:
      lb = elbs[0]
  except Exception:
    port = relationDestinationPort()

    lb = elb.create_load_balancer(
      name,
      getAvailabilityZoneList(),
      [ (charm_config['http_port'],port,'HTTP') ]
    )
  return lb

def relationDestinationPort():
  return 8000

def updateListeners(lb):
  port = relationDestinationPort()
  if port == None:
    hookenv.log("No relations to setup")
    return

  hookenv.log("Setup up ELB to destination port: %s" % port)
  expected_listeners = [
    (charm_config['http_port'],port,'HTTP')
  ]
  hasSSL = False
  if charm_config['ssl_cert_id'] != "":
    hasSSL = True
    expected_listeners.append( (charm_config['ssl_port'],port,'HTTPS') )

  missing_listeners = []
  invalid_listeners = []

  for listener in lb.listeners:
    try:
      found = expected_listeners.index( listener )
      expected_listeners.remove(found)
    except ValueError:
      invalid_listeners.append(listener)

  # remove invalid listeners
  if len(invalid_listeners) > 0:
    hookenv.log('Removing invalid listeners')
    for invalid_listener in invalid_listeners:
      lb.delete_listener(invalid_listener[0])

  # add missing listeners
  if len(missing_listeners) > 0:
    hookenv.log('Adding missing listeners')
    lb.create_listeners(missing_listeners)

  # set ssl cert id on ssl port if required
  if hasSSL == True:
    hookenv.log("Setting SSL Certificate ID on SSL Port")
    lb.set_listener_SSL_certificate( charm_config['ssl_port'], charm_config['ssl_cert_id'] )

def updateAvailabilityZones(lb):
  # az = getAvailabilityZoneList()
  pass

def updateInstances(lb):
  hookenv.log("Update Instances")

  instances = []
  ec2 = getEC2()
  relations = hookenv.relations_of_type('reverseproxy')
  for relation in relations:
    filter = {
      "private-dns-name": relation['hostname']
    }
    instances = ec2.get_all_instances(filters=filter)
    if len(instances) > 0:
      instances.append(instances[0]['instances'][0]['id'])
    else:
      hookenv.log("Missing instance for relation, possibly configure in wrong region?")

  print (lb.instances,instances)



hooks = hookenv.Hooks()

@hooks.hook("config-changed")
def config_changed():
  hookenv.log("config changed")
  lb = findOrCreateELB(charm_config['elb_name'])
  updateListeners(lb)
  updateInstances(lb)

@hooks.hook("reverseproxy-relation-joined")
def reverseproxy_relation_joined():
  hookenv.log("reverseproxy relation joined")
  lb = findOrCreateELB(charm_config['elb_name'])
  updateInstances(lb)

@hooks.hook("reverseproxy-relation-changed")
def reverseproxy_relation_changed():
  hookenv.log("reverseproxy relation changed")
  lb = findOrCreateELB(charm_config['elb_name'])
  updateInstances(lb)

@hooks.hook("reverseproxy-relation-departed")
def reverseproxy_relation_departed():
  hookenv.log("reverseproxy relation departed")
  lb = findOrCreateELB(charm_config['elb_name'])
  updateInstances(lb)

@hooks.hook("reverseproxy-relation-broken")
def reverseproxy_relation_broken():
  hookenv.log("reverseproxy relation broken")
  lb = findOrCreateELB(charm_config['elb_name'])
  updateInstances(lb)

if __name__ == "__main__":
  # execute a hook based on the name the program is called by
  hooks.execute(sys.argv)

